@@ -1,7 +1,7 @@
 # -*- mode: Perl -*-
 package MRTG_lib;
 ###################################################################
-# MRTG 2.14.7  Support library MRTG_lib.pm
+# MRTG 2.15.1  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <tobi@oetiker.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -99,7 +99,9 @@ $VERSION = 2.100016;
        [sub{((lc($_[0])) eq 'yes' or (lc($_[0])) eq 'no')}, sub{"$_[0] must be yes or no"}],
 
        'interval' => 
-       [sub{int($_[0]) >= 1 and int($_[0]) <= 60}, sub{"$_[0] should be at least 1 Minute and no more than 60 Minutes"}], 
+       [sub{$_[0] =~ /(\d+)(?::(\d+))?/ ; 
+            my $int = $1*60; $int += $2 if $2;
+            $int >= 1 and $int <= 60*60}, sub{"$_[0] should be at least 1 Second (0:01) and no more than 60 Minutes (60)"}], 
 
        'writeexpires' =>  
        [sub{1}, sub{"Internal Error"}],
@@ -314,6 +316,16 @@ $VERSION = 2.100016;
 
        'threshdir' =>
        [sub{$_[0] && (-d $_[0])}, sub{"Threshold directory $_[0] does not exist"}],
+ 
+       'threshhyst' =>
+       [sub{($_[0] =~ /^[0-9]+(?:\.[0-9]+)?$/)},
+        sub{"$_[0] must be a Number xxx.xxx"}],
+
+       'threshmailserver' =>
+       [sub{$_[0] && gethostbyname($_[0])}, sub{"Unknown mailserver hostname $_[0]"}],
+
+       'threshmailsender' =>
+       [sub{$_[0] && ($_[0] =~ /\S+\@\S+/)}, sub{"ThreshMailAddress $_[0] does not look like an email address at all"}],
 
        'threshmini[]' =>
        [sub{1}, sub{"Internal Threshold Config Error"}],
@@ -342,6 +354,9 @@ $VERSION = 2.100016;
        'threshprogoko[]' =>
        [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
 
+       'threshmailaddress[]' =>
+       [sub{$_[0] && ($_[0] =~ /\S+\@\S+/)}, sub{"ThreshMailAddress $_[0] does not look like an email address at all"}],
+
        'timestrpos[]' => 
        [sub{$_[0] =~ /^(no|[lr][ul])$/i}, sub{"Must be a string of NO, LU, RU, LL, RL"}],
 
@@ -657,6 +672,16 @@ sub cfgcheck ($$$$;$) {
           mkdirhier $$cfg{imagedir}  unless $opts->{check};
         }
     }
+    if ($cfg->{threshmailserver} and not $cfg->{threshmailsender}){
+	warn ("WARNING: If \"ThreshMailServer\" is defined, then \"ThreshMailSender\" must be defined too.\n");
+        $error = "yes";
+    }
+    if ($cfg->{threshmailsender} and not $cfg->{threshmailserver}){
+	warn ("WARNING: If \"ThreshMailSender\" is defined, then \"ThreshMailServer\" must be defined too.\n");
+        $error = "yes";
+    }
+ 
+    $cfg->{threshhyst} = 0.1 unless $cfg->{threshhyst};
     # build relativ path from htmldir to image dir.
     my @htmldir = split /\Q${MRTG_lib::SL}\E+/, $$cfg{htmldir};
     my @imagedir =  split /\Q${MRTG_lib::SL}\E+/, $$cfg{imagedir};
@@ -735,10 +760,15 @@ sub cfgcheck ($$$$;$) {
     }
 
     # default interval is 5 minutes
-    $$cfg{interval} = 5 unless defined $$cfg{interval};
+    if ($cfg->{interval} and $cfg->{interval} =~ /(\d+)(?::(\d+))?/){
+	$cfg->{interval} = $1;
+	$cfg->{interval} += $2/60.0 if $2;
+    } else {
+        $cfg->{interval} = 5;
+    }
     unless ($$cfg{logformat} eq 'rrdtool') {
         # interval has to be 5 minutes at least without userrdtool
-        if ($$cfg{interval} < 5) {
+        if ($$cfg{interval} < 5.0) {
             die "ERROR: CFG Error in \"Interval\": should be at least 5 Minutes (unless you use rrdtool)";
         }
     }
@@ -755,10 +785,18 @@ sub cfgcheck ($$$$;$) {
         die "ERROR: File $cfg->{ conversioncode } conversion code evaluation failed\n$@\n"
             unless eval $code;
     }
+    my $thresh_error;
 
     foreach $rou (@$routers) {
         # and now for the testing
-
+	if (defined $rcfg->{threshmailaddress}{$rou}){
+	    if (not defined  $cfg->{threshmailserver} and not $thresh_error){
+		warn (qq{ERROR: ThreshMailAddress[$rou]: specified without "ThreshMailServer:"});
+		$error = "yes";
+		$thresh_error = "yes";
+            }
+	    # the dependency between sender and server is taken care of already
+	}	
 	if (! defined $rcfg->{snmpoptions}{$rou}) {
 		$rcfg->{snmpoptions}{$rou} = {%{$cfg->{snmpoptions}}}
 		  if defined $cfg->{snmpoptions};
@@ -1030,7 +1068,7 @@ sub datestr ($) {
 # create expire date for expiery in ARG Minutes
 
 sub expistr ($) {
-    my ($time) = time+$_[0]*60+5;
+    my ($time) = time+int($_[0]*60)+5;
     my ($wday) = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat')[(gmtime($time))[6]];
     my ($month) = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep', 
                    'Oct','Nov','Dec')[(gmtime($time))[4]];
@@ -1114,6 +1152,7 @@ sub demonize_me ($) {
               require 'POSIX.pm';
               POSIX::setsid() or die "Can't start a new session: $!";
               open STDOUT,'>/dev/null' or die "ERROR: Redirecting STDOUT to /dev/null: $!";
+              open STDERR,'>/dev/null' or die "ERROR: Redirecting STDERR to /dev/null: $!";
               open STDIN, '</dev/null' or die "ERROR: Redirecting STDIN from /dev/null: $!";
       }
    }
@@ -1282,7 +1321,7 @@ sub targparser( $$$$$ ) {
 	# will match. Therefore the match must be postprocessed to check that
 	# $ifOID and $ifAlt are not both null.
 	my $ifComplex =
-		'([-a-zA-Z0-9]*(?:\.\d+)*?)' .	# OID possibly starting with a MIB name ($if->{OID})
+		'((?:\.\d+)*?\.?[-a-zA-Z0-9]*(?:\.\d+)*?)' .	# OID possibly starting with a MIB name ($if->{OID})
 		'(' .							# Interface specification alternatives: ($if->{Alt})
 			'\.' .						#  separator
 			$ifSimple .					#  simple alternatives (6 variables)
@@ -1736,7 +1775,7 @@ sub log2rrd ($$$) {
 	$DST = 'COUNTER'
     }
 
-    my $MHB = $$cfg{interval} * 60 * 2;
+    my $MHB = int($$cfg{interval} * 60 * 2);
 
     my $MAX1 =
       $$rcfg{'absmax'}{$router}
@@ -1864,7 +1903,7 @@ sub debug ($$) {
     my $tag = shift;
     my $msg = shift;
     return unless grep {$_ eq $tag} @main::DEBUG;
-    warn "--",$tag,": ",$msg,"\n";
+    warn "--".$tag.": ".$msg."\n";
     return;
 }
 
